<?php
/**
 * Magento
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@magento.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Magento to newer
 * versions in the future. If you wish to customize Magento for your
 * needs please refer to http://www.magento.com for more information.
 *
 * @category    Mage
 * @package     Mage_CatalogSearch
 * @copyright  Copyright (c) 2006-2015 X.commerce, Inc. (http://www.magento.com)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

/**
 * Layered Navigation block for search
 *
 */
class Semknox_ProductSearch_Model_CatalogSearch_Layer_Filter_Item //Mage_Catalog_Block_Layer_View
{

    protected $item;
    protected $filterId;

    public function __construct($args)
    {

        $this->item = $args['item'];
        $this->filterId = $args['filterId'];

    }

    public function getCount()
    {
        return $this->item['count'];
    }

    public function getId()
    {
        return $this->item['id'];
    }

    public function getUrl()
    {
        $query = Mage::app()->getRequest()->getParams();

        if(isset($query['filter'][$this->filterId]) && is_array($query['filter'][$this->filterId]) && in_array($this->getId(), $query['filter'][$this->filterId]))
        {
            $query['filter'][$this->filterId] = array_diff($query['filter'][$this->filterId], array($this->getId()));
        } else {
            $query['filter'][$this->filterId][] = $this->getId();
        }


        /* @var Semknox_ProductSearch_Helper_Data $helper */
        $helper = Mage::helper('semknoxps');

        if($helper->semknoxSeoUrlIsActive()) {

            $slug = $helper->getLastDirectoryInUrl();

        } else {

            $slug = $helper->getResultsUrl();
        }

        $url = Mage::getUrl($slug, array(
            '_query' => $query
        ));

        return $url;
    }

    public function getItemIsActive()
    {
        return isset($this->item['set']);
    }

    public function getIconSrc()
    {
        if(isset($this->item['icon']) && strlen($this->item['icon']))
            return $this->item['icon'];

        return false;
    }

    public function getLabel()
    {
        $class = 'filter_'.$this->filterId;
        $class .= ' option_'.$this->getId();
        $class .= $this->getItemIsActive() ? ' active' : '';

        $dataHover = ($this->getIconSrc()) ? ' data-hover="'.$this->getIconSrc().'" ' : '';

        $dataValue = ' data-sxvalue="'.$this->filterId.'" ';
        $dataOption = ' data-sxoption="'.$this->getId().'" ';

        return '<span class="'.$class.'" '.$dataHover.$dataValue.$dataOption.' data-sxkey="filter">'.$this->item['viewName'].'</span>';
    }


}
